;     Guile Scheme Debconf interface
;     Copyright (C) 2002 Clinton Ebadi

;     This program is free software; you can redistribute it and/or modify
;     it under the terms of the GNU General Public License as published by
;     the Free Software Foundation; either version 2 of the License, or
;     (at your option) any later version.

;     This program is distributed in the hope that it will be useful,
;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;     GNU General Public License for more details.

;     You should have received a copy of the GNU General Public License
;     along with this program; if not, write to the Free Software
;     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


;;; I read the Debconf standard (look in the debian-policy package)
;;; and tried to make this do what it says to do

;;; TODO: 
;;; * make calls cleane (e.g. (define pri-medium "medium"))

(define-module (debconf)
  #:use-module (ice-9 rdelim)
  #:use-module (ice-9 popen)
  #:use-module (srfi srfi-1)
  #:export (output-text capb set reset title input beginblock
			endblock go get register unregister 
			subst previous-module fset fget purge 
			metaget dversion clear write-text pri-low 
			pri-medium pri-high pri-critical dselect 
			multiselect stop))

;;; initalize debconf
(if (not (= (getuid) 0))
  (error "Not Running as Root! Debconf cannot continue...exiting\n"))

;;; restart our script from the frontend
(if (not (getenv "DEBIAN_HAS_FRONTEND"))
  (apply execl `("/usr/share/debconf/frontend"
		 ,(car (command-line))
		 ,@(command-line))))

;;; okay, so I guess we can assume that were are being run from the
;;; frontend because $DEBIAN_HAS_FRONTEND is set in the environment

;;; export these? Maybe someone might want to do Debconf hackery...
(define debconf-out #f)
(define debconf-in #f)

(set! debconf-out (current-output-port))
(set! debconf-in (current-input-port))

;;; redir output (BROKEN)
;;; I don't think this works quite right (not safe if called more than
;;; once) or maybe not because modules are only loaded once in Guile.
;;; Can't hurt though...
;;; Now c-err and c-out are the same port (debconf-out is now the old c-out)

;;; ok, this breaks everything :) let's just take a little risk

;(define debconf-redir #f)
;(cond ((not debconf-redir)
;       (set-current-output-port (current-error-port))
;       (set! debconf-redir #t)))


;;; support functions
(define (%dout . args)
  (let ((str (apply string-append args)))
    (write-line str)
    (force-output debconf-out))) ; we MUST FORCE output

(define (%read-line)
  (read-line debconf-in))

(define (%success? code) ; this determines if code = success
  (let loop ((success-codes '(0 30)))
    (cond ((null? success-codes) #f)
	  ((= code (car success-codes)) #t)
	  (else (loop (cdr success-codes))))))

(define (%check-error)
  (let ((line (fold-string (%read-line))))
    (if (%success? (string->number (car line)))
      line ; the caller might need this (see capb)
      (begin
	(warn "Something wierd happened with Debconf"
	      "Attempting to continue."
	      "Error code was:"
	      (car line)
	      "Error message was:"
	      (if (null? (cdr line))
		"No error mesage returned"
		(cdr line)))
	line )))) ; we return line to let the caller extract err code

(define (%err-code err-list)
  (string->number (car err-list)))
    
(define (%list->space-string lst)
  (let loop ((lst (cdr lst))
	     (str (car lst)))
    (if (null? lst)
      str
      (loop (cdr lst) (string-append str " " (car lst))))))

(define (fold-string str)
  (let ((str-list (string-split str #\space)))
    (let loop ((str-list str-list)
	       (new-list (list)))
      (if (null? str-list)
	(reverse new-list)
	(if (string=? (car str-list) "")
	  (loop (cdr str-list) new-list)
	  (loop (cdr str-list) (cons (car str-list) new-list)))))))


;;; OK, now onto the meat of the interface. I could use a macro
;;; generate functions like some of the other language interfaces, but
;;; I don't want to do that because there is less checking of
;;; arguments. Guile should produce nice helpful error message while
;;; having Debconf die isn't so nice...

;;; definitions that make debconf easier to use
;; priorities
(define pri-low "low")
(define pri-medium "medium")
(define pri-high "high")
(define pri-critical "critical")


;;; functions

;; all guile-debconf scripts should use write-text because it will
;; send the text to the correct place DO NOT USE DISPLAY
(define (write-text . args)
  (for-each 
   (lambda (x) (if (procedure? x)
		 (display (x) (current-error-port))
		 (display x (current-error-port))))
   args))

(define (dversion number) ; version conflicts with version :)
  (%dout "VERSION " number)
  (%err-code (%check-error)))

;;; I'm a bit fuzzy on what capb does exactly
(define (capb . capabilities)
  (%dout "CAPB " (%list->space-string capabilities))
  (cdr (%check-error))) ; return capabilities

(define (title tit)
  (%dout "TITLE " tit)
  (%err-code (%check-error)))

(define (stop)
  (%dout "STOP"))

(define (input pri question?)
  (%dout "INPUT " pri " " question?)
  (%err-code (%check-error)))

(define (beginblock)
  (%dout "BEGINBLOCK")
  (%err-code (%check-error)))

(define (endblock)
  (%dout "ENDBLOCK")
  (%err-code (%check-error)))

(define (go) ; go is special (wierd return values, deal with it)
  (%dout "GO")
  (let* ((line (fold-string (%read-line)))
	 (retval (string->number (car line))))
    (if (or (= retval 0) (= retval 30))
      retval
      -1 )))

(define (clear)
  (%dout "CLEAR")
  (%err-code (%check-error)))

(define (get question?)
  (%dout "GET " question?)
  (%list->space-string (cdr (%check-error))))

(define (set question? value)
  (%dout "SET " question? " " value)
  (%err-code (%check-error)))

(define (reset question?)
  (%dout "RESET " question?)
  (%err-code (%check-error)))

(define (subst question? key value)
  (%dout "SUBST " question? " " key " " value)
  (%err-code (%check-error)))

(define (fget question? flag)
  (%dout "FGET " question? " " flag)
  (let ((ret (second (%check-error))))
    (if (string=? ret "true")
      #t
      #f )))

(define (fset question? flag value)
  (%dout "FSET " question? " "
	 (if flag
	   "true"
	   "false" ) " " value)
  (%err-code (%check-error)))

(define (metaget question? field)
  (%dout "METAGET " question? " " field)
  (%list->space-string (cdr (%check-error))))

(define (register template question?)
  (%dout "REGISTER " template " " question?)
  (%err-code (%check-error)))

(define (unregister question?)
  (%dout "UNREGISTER " question?)
  (%err-code (%check-error)))

(define (purge)
  (%dout "PURGE ")
  (%err-code (%check-error)))

(define (dselect . args)
  (%dout "SELECT " (%list->space-string args))
  (%error-code (%check-error)))

(define (multiselect . args)
  (%dout "MULTISELECT " (%list->space-string args))
  (%error-code (%check-error)))

;;; now test this when I get back!